@@ -8,7 +8,7 @@
  ***************************************************************************/
 
  static char *const _id =
-"$Id: lockfile.c,v 1.36 2002/08/06 19:14:14 papowell Exp $";
+"$Id: lockfile.c,v 1.41 2002/12/04 21:12:17 papowell Exp $";
 
 /***************************************************************************
  * MODULE: lockfile.c
@@ -144,6 +144,85 @@ int Do_lock( int fd, int block )
 
 
 
+/***************************************************************************
+ * Do_unlock( fd )
+ * unlocks a lock on a file;
+ * Returns: < 0 if lock fn failed
+ *            0 if successful
+ ***************************************************************************/
+
+int Do_unlock( int fd )
+{
+    int code = -2;
+
+	DEBUG3("Do_unlock: fd %d", fd );
+
+#if defined(HAVE_FLOCK)
+	if( code == -2 ){
+		int err;
+		int how;
+
+		how = LOCK_EX|LOCK_UN;
+		DEBUG3 ("Do_unlock: using flock" );
+		code = flock( fd, how );
+		err = errno;
+		if( code < 0 ){
+			DEBUG1( "Do_unlock: flock failed '%s'", Errormsg( err ));
+			code = -1;
+		} else {
+			code = 0;
+		}
+		errno = err;
+	}
+#endif
+#if defined(HAVE_LOCKF)
+	if( code == -2 ){
+		int err;
+		int how;
+
+		how = F_ULOCK;
+
+		DEBUG3 ("Do_unlock: using lockf" );
+		code = lockf( fd, how, 0);
+		err = errno;
+		if( code < 0 ){
+			DEBUG1( "Do_unlock: lockf failed '%s'", Errormsg( err));
+			code = -1;
+		} else {
+			code = 0;
+		}
+		errno = err;
+	}
+#endif
+#if defined(HAVE_FCNTL)
+	if( code == -2 ){
+		struct flock file_lock;
+		int err;
+		int how;
+		DEBUG3 ("Do_unlock: using fcntl with SEEK_SET" );
+
+		how = F_SETLK;
+		memset( &file_lock, 0, sizeof( file_lock ) );
+		file_lock.l_type = F_UNLCK;
+		file_lock.l_whence = SEEK_SET;
+		code = fcntl( fd, how, &file_lock);
+		err = errno;
+		if( code < 0 ){
+			code = -1;
+		} else {
+			code = 0;
+		}
+		DEBUG3 ("devlock_fcntl: status %d", code );
+		errno = err;
+	}
+#endif
+
+	DEBUG3 ("Do_unlock: status %d", code);
+	return( code);
+}
+
+
+
 
 /***************************************************************************
  * LockDevice(fd, block)
