@@ -4,7 +4,7 @@
  * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
- * $Id: lpd_rcvjob.h,v 1.36 2002/08/06 19:14:16 papowell Exp $
+ * $Id: lpd_rcvjob.h,v 1.41 2002/12/04 21:12:19 papowell Exp $
  ***************************************************************************/
 
 
@@ -20,8 +20,8 @@ int Read_one_line( int fd, char *buffer, int maxlen );
 int Check_space( double jobsize, int min_space, char *pathname );
 int Do_perm_check( struct job *job, char *error, int errlen );
 int Check_for_missing_files( struct job *job, struct line_list *files,
-	char *error, int errlen, struct line_list *header_info, int holdfile_fd );
-int Set_up_temporary_control_file( struct job *job,
+	char *error, int errlen, struct line_list *header_info, int *holdfile_fd );
+int Set_up_temporary_hold_file( struct job *job,
 	char *error, int errlen  );
 int Find_non_colliding_job_number( struct job *job );
 int Get_route( struct job *job, char *error, int errlen );
